@@ -1,4 +1,4 @@
-# $Id: DST.pm,v 1.4 2004/09/21 20:33:10 epaepa Exp $
+# $Id: DST.pm,v 1.6 2008/10/28 14:46:13 epaepa Exp $
 #
 # Timezone stuff, including routines to guess timezones in European
 # (and other) countries that have daylight saving time.
@@ -76,9 +76,12 @@ our @EXPORT = qw(parse_local_date date_to_local utc_offset);
 sub parse_local_date($$) {
 #    local $Log::TraceMessages::On = 1;
     my ($date, $base) = @_;
-    croak 'usage: parse_local_date(unparsed date, base timezone)'
+    croak 'usage: parse_local_date(unparsed date, base timeoffset)'
       if @_ != 2 or not defined $date or not defined $base;
-    my $winter_tz = tz_to_num($base);
+    croak 'second parameter must be a time offset (+xxxx,-xxxx)'
+      if( $base !~ /^[-+]\d{4}$/ );
+
+    my $winter_tz = $base;
     my $summer_tz = sprintf('%+05d', $winter_tz + 100); # 'one hour'
 
     my $got_tz = gettz($date);
@@ -259,6 +262,9 @@ sub utc_offset( $$ ) {
     my ($indate, $basetz) = @_;
     croak "empty date" if not defined $indate;
     croak "empty base TZ" if not defined $basetz;
+    $basetz = tz_to_num( $basetz ) 
+      if $basetz !~ /^[-+]\d{4}$/;
+
     my $d = date_to_local(parse_local_date($indate, $basetz), $basetz);
     return UnixDate($d->[0],"%Y%m%d%H%M%S") . " " . $d->[1];
 }
@@ -308,6 +314,13 @@ sub dst_dates_na( $$ ) {
     die "usage: dst_dates(year, winter_tz), got args: @_" if @_ != 2;
     my ($year, $winter_tz) = @_;
     die "don't know about DST before 1988" if $year < 1988;
+    return dst_dates_na_old($year, $winter_tz) if $year < 2007;
+    return dst_dates_na_new($year, $winter_tz);
+}
+
+# Old North American daylight saving time, used before 2007.
+sub dst_dates_na_old( $$ ) {
+    my ($year, $winter_tz) = @_;
     $winter_tz =~ /^\s*-\s*(\d\d)(?:00)?\s*$/
       or die "bad North American winter time zone $winter_tz";
     my $hours = $1;
@@ -340,6 +353,49 @@ sub dst_dates_na( $$ ) {
     return [ $start_dst, $end_dst ];
 }
 
+# New US daylight saving time from 2007, also followed by most
+# Canadian provinces.
+#
+sub dst_dates_na_new( $$ ) {
+    my ($year, $winter_tz) = @_;
+    $winter_tz =~ /^\s*-\s*(\d\d)(?:00)?\s*$/
+      or die "bad North American winter time zone $winter_tz";
+    my $hours = $1;
+
+    my ($start_dst, $end_dst);
+    my $seen_Sunday_in_March = 0;
+    foreach (1 .. 31) {
+	if (not defined $start_dst) {
+	    my $date = "$year-03-$_";
+	    my $day = UnixDate(parse_date($date), '%A');
+	    if ($day =~ /Sunday/) {
+		if ($seen_Sunday_in_March) {
+		    # Second Sunday in March.  DST starts at 02:00
+		    # local standard time.
+		    #
+		    $start_dst = Date_ConvTZ(parse_date("$date 02:00"),
+					     "-$winter_tz", 'UTC');
+		}
+		else {
+		    $seen_Sunday_in_March = 1;
+		}
+	    }
+	}
+
+	next if defined $end_dst;
+	my $date = "$year-11-$_";
+	my $day = UnixDate(parse_date($date), '%A');
+	next unless $day =~ /Sunday/;
+	# A Sunday in November (and the first one we see).  DST ends
+	# at 01:00 local standard time.
+	#
+	$end_dst = Date_ConvTZ(parse_date("$date 01:00"),
+			       "-$winter_tz", 'UTC');
+    }
+    die if not defined $start_dst or not defined $end_dst;
+
+    return [ $start_dst, $end_dst ];
+}
 
 
 1;
